/* Name: main.c
 * Project: HID-Test
 * Author: Christian Starkjohann
 * Creation Date: 2006-02-02
 * Tabsize: 4
 * Copyright: (c) 2006 by OBJECTIVE DEVELOPMENT Software GmbH
 * License: Proprietary, free under certain conditions. See Documentation.
 */

#define F_CPU   12000000L  

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <avr/delay.h>
#include <string.h>

#include "usbdrv.h"
#include "oddebug.h"
#include "gamepad.h"
#include "snes.h"
#include "nes.h"
#include "leds.h"


static Gamepad *curGamepad;


/* ----------------------- hardware I/O abstraction ------------------------ */

static void hardwareInit(void)
{
	uchar	i, j;

	// init port C as input with pullup
	PORTC = 0xff;
	DDRC = 0x00;

	PORTD = 0xfa;   /* 1101 1010 bin: activate pull-ups except on USB lines */
	DDRD = 0x07;    /* 0010 0111 bin: all pins input except USB (-> USB reset) and led */
	j = 0;
	while(--j){     /* USB Reset by device only required on Watchdog Reset */
		i = 0;
		while(--i); /* delay >10ms for USB reset */
	}
	DDRD = 0x02;    /* 0000 0010 bin: remove USB reset condition */
			/* configure timer 0 for a rate of 12M/(1024 * 256) = 45.78 Hz (~22ms) */
	TCCR0 = 5;      /* timer 0 prescaler: 1024 */

}

static uchar    reportBuffer[2];    /* buffer for HID reports */



/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

static uchar    idleRate;           /* in 4 ms units */


uchar	usbFunctionSetup(uchar data[8])
{
	usbRequest_t    *rq = (void *)data;

	usbMsgPtr = reportBuffer;
	if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS){    /* class request type */
		if(rq->bRequest == USBRQ_HID_GET_REPORT){  /* wValue: ReportType (highbyte), ReportID (lowbyte) */
			/* we only have one report type, so don't look at wValue */
			curGamepad->buildReport(reportBuffer);
			return curGamepad->report_size;
		}else if(rq->bRequest == USBRQ_HID_GET_IDLE){
			usbMsgPtr = &idleRate;
			return 1;
		}else if(rq->bRequest == USBRQ_HID_SET_IDLE){
			idleRate = rq->wValue.bytes[1];
		}
	}else{
	/* no vendor specific requests implemented */
	}
	return 0;
}

/* ------------------------------------------------------------------------- */

static void blinkLed(int n_times)
{
	int i;

	LED_OFF();
	while (n_times--)
	{
		LED_ON();
		for (i=0; i<20; i++) _delay_ms(10);
		LED_OFF();
		for (i=0; i<50; i++) _delay_ms(10);
	}
}

int main(void)
{
	char must_report = 0, first_run = 1;
	uchar   idleCounter = 0;
	int run_mode;

	// led pin as output
	DDRD |= 0x20;

	/* Dip switch common: DB0, outputs: DB1 and DB2 */
	DDRB |= 0x01;
	DDRB &= ~0x06; 
	
	PORTB |= 0x06; /* enable pull up on DB1 and DB2 */
	PORTB &= ~0x01; /* Set DB0 to low */

	_delay_ms(10); /* let pins settle */

	run_mode = (PINB & 0x06)>>1;

	switch(run_mode)
	{
		default:
		case 0:
		case 1:
			while(1)
				blinkLed(1);
			break;
		case 2:	/* DB1 on, DB2 off. SNES mode */
			curGamepad = nesGetGamepad();
			blinkLed(1);
			break;
		case 3:
			curGamepad = snesGetGamepad();
			blinkLed(2);
			break;
	}
	

	// configure report descriptor according to
	// the current gamepad
	rt_usbHidReportDescriptor = curGamepad->reportDescriptor;
	rt_usbHidReportDescriptorSize = curGamepad->reportDescriptorSize;
	

	//wdt_enable(WDTO_2S);
	hardwareInit();
	curGamepad->init();
	odDebugInit();
	usbInit();
	sei();
	DBG1(0x00, 0, 0);

	
	for(;;){	/* main event loop */
		wdt_reset();

		// this must be called at each 50 ms or less
		usbPoll();

		if (first_run) {
			curGamepad->update();
			first_run = 0;
		}

		if(TIFR & (1<<TOV0)){   /* 22 ms timer */
			TIFR = 1<<TOV0;
			if(idleRate != 0){
				if(idleCounter > 4){
					idleCounter -= 5;   /* 22 ms in units of 4 ms */
				}else{
					idleCounter = idleRate;
					must_report = 1;
				}
			}
		
			// read the controller. On a real Snes, this
			// is done every 16.67 ms (60hz)
			curGamepad->update();
		}
 
		
		if (curGamepad->changed()) {
//			LED_TOGGLE();
			must_report = 1;
		}
		
		if(must_report && usbInterruptIsReady()){
			must_report = 0;

			curGamepad->buildReport(reportBuffer);
			usbSetInterrupt(reportBuffer, curGamepad->report_size);
		}
	}
	return 0;
}

/* ------------------------------------------------------------------------- */
